// 
// Copyright (C) 2007, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Sends an alert to the alert window when a price level has been reached.
	/// </summary>
	[Description("Sends an alert to the alert window when a price level is BELOW your level has been reached.")]
	public class PriceAlertNeg : Indicator
	{
		#region Variables
		private double					price						= 0;
		private bool					showTriggerLine				= true;
		private bool					triggered					= false;
		private bool					triggerOnGreaterThan		= false;
		private bool					triggerSet					= false;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Navy, PlotStyle.Line, "Trigger line"));

			ChartOnly			= true;
			AutoScale			= false;
			CalculateOnBarClose = false;
			DisplayInDataBox	= false;
			Overlay				= true;
			PriceTypeSupported	= false;
		}
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (ShowTriggerLine)
				Values[0].Set(Price);

			if (Historical || triggered)
				return;
			else if (!triggerSet)
			{
				triggerOnGreaterThan = Close[0] <= price - (TickSize * 0.5) ? false : true;
				triggerSet = true;
			}

			if ((triggerOnGreaterThan && Close[0] <= Price - (TickSize * 0.5)) || (!triggerOnGreaterThan && Close[0] <= Price + (TickSize * 0.5)))
			{
				triggered = true;
				Alert(DateTime.Now.Millisecond.ToString(), NinjaTrader.Cbi.Priority.Medium, "Price level '" + Price + "' hit!", Cbi.Core.InstallDir + @"\sounds\Alert4.wav", 0, Color.Yellow, Color.Black);
			}
		}


		#region Properties
		/// <summary>
		/// </summary>
		[Description("A NEGATIVE price alert level.")]
		[Category("Parameters")]
		public double Price
		{
			get { return price; }
			set { price = Math.Min(0, value); }
		}

		[Browsable(true)]
		[Gui.Design.DisplayNameAttribute("Show trigger line")]
		[Category("Parameters")]
		public bool ShowTriggerLine
		{
			get { return showTriggerLine; }
			set { showTriggerLine = value; }
		}
		#endregion
	}
}



#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private PriceAlertNeg[] cachePriceAlertNeg = null;

        private static PriceAlertNeg checkPriceAlertNeg = new PriceAlertNeg();

        /// <summary>
        /// Sends an alert to the alert window when a price level is BELOW your level has been reached.
        /// </summary>
        /// <returns></returns>
        public PriceAlertNeg PriceAlertNeg(double price, bool showTriggerLine)
        {
            return PriceAlertNeg(Input, price, showTriggerLine);
        }

        /// <summary>
        /// Sends an alert to the alert window when a price level is BELOW your level has been reached.
        /// </summary>
        /// <returns></returns>
        public PriceAlertNeg PriceAlertNeg(Data.IDataSeries input, double price, bool showTriggerLine)
        {
            checkPriceAlertNeg.Price = price;
            price = checkPriceAlertNeg.Price;
            checkPriceAlertNeg.ShowTriggerLine = showTriggerLine;
            showTriggerLine = checkPriceAlertNeg.ShowTriggerLine;

            if (cachePriceAlertNeg != null)
                for (int idx = 0; idx < cachePriceAlertNeg.Length; idx++)
                    if (Math.Abs(cachePriceAlertNeg[idx].Price - price) <= double.Epsilon && cachePriceAlertNeg[idx].ShowTriggerLine == showTriggerLine && cachePriceAlertNeg[idx].EqualsInput(input))
                        return cachePriceAlertNeg[idx];

            PriceAlertNeg indicator = new PriceAlertNeg();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Price = price;
            indicator.ShowTriggerLine = showTriggerLine;

            PriceAlertNeg[] tmp = new PriceAlertNeg[cachePriceAlertNeg == null ? 1 : cachePriceAlertNeg.Length + 1];
            if (cachePriceAlertNeg != null)
                cachePriceAlertNeg.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cachePriceAlertNeg = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Sends an alert to the alert window when a price level is BELOW your level has been reached.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PriceAlertNeg PriceAlertNeg(double price, bool showTriggerLine)
        {
            return _indicator.PriceAlertNeg(Input, price, showTriggerLine);
        }

        /// <summary>
        /// Sends an alert to the alert window when a price level is BELOW your level has been reached.
        /// </summary>
        /// <returns></returns>
        public Indicator.PriceAlertNeg PriceAlertNeg(Data.IDataSeries input, double price, bool showTriggerLine)
        {
            return _indicator.PriceAlertNeg(input, price, showTriggerLine);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Sends an alert to the alert window when a price level is BELOW your level has been reached.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PriceAlertNeg PriceAlertNeg(double price, bool showTriggerLine)
        {
            return _indicator.PriceAlertNeg(Input, price, showTriggerLine);
        }

        /// <summary>
        /// Sends an alert to the alert window when a price level is BELOW your level has been reached.
        /// </summary>
        /// <returns></returns>
        public Indicator.PriceAlertNeg PriceAlertNeg(Data.IDataSeries input, double price, bool showTriggerLine)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.PriceAlertNeg(input, price, showTriggerLine);
        }

    }
}
#endregion
